@@ -24,4 +24,6 @@ class bench(sub4):
 	def oneway(self, *args):
 		# oneway doesn't return anything
 		pass
+	def mapping(self, mapping):
+		return mapping
 
